/*
 * Copyright (c) 1998-2002 The Jgroup Team.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package jgroup.test.gm;

import jgroup.core.GroupManager;
import jgroup.core.JgroupException;
import jgroup.core.MemberId;
import jgroup.core.MembershipListener;
import jgroup.core.MembershipService;
import jgroup.core.View;
import jgroup.relacs.config.AppConfig;

/**
 *  Jgroup server for measuring the performance of IGMI invocations versus
 *  message multicasing.
 *
 *  @author Hein Meling
 *  @since Jgroup 1.2
 */
public class RemoteTestServer
  implements MembershipListener
{

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Fields (non-shared state)
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** Keep the time of the last view change */
  private long lastViewChange = 0;

  /** Buffer for storing log output */
  private StringBuilder buf = new StringBuilder();

  private RemoteTestService rts;

  private MemberId myId;

  private int minRedundancy;


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constructor
  ////////////////////////////////////////////////////////////////////////////////////////////

  public RemoteTestServer()
    throws JgroupException
  {
    buf.append("----------------------------------------------------------\n");
    buf.append("Log time : Number of members : Time since last view change\n");

    GroupManager gm = GroupManager.getGroupManager(this);
    rts = (RemoteTestService) gm.getService("RemoteTest");
    MembershipService pgms = (MembershipService) gm.getService("PGMS");
    AppConfig thisApp = AppConfig.getApplication(this);
    minRedundancy = thisApp.getMinimalRedundancy();
    myId = pgms.getMyIdentifier();
    pgms.join(thisApp.getGroupId());
    System.out.println("RemoteTestServer initialized");
  }


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods from MembershipListener
  ////////////////////////////////////////////////////////////////////////////////////////////

  public void viewChange(View view)
  {
    long currentTime = System.currentTimeMillis();

    buf.append(currentTime);
    buf.append(" : ");
    buf.append(view.size());
    buf.append(" : ");
    buf.append((lastViewChange != 0) ? (currentTime - lastViewChange) : 0);
    buf.append("\n");
    System.out.println(buf.toString());

    /* Keep the time of this view change */
    lastViewChange = currentTime;

    System.out.println("min:" +minRedundancy);
    System.out.println("myid:" +myId);

    /*
     * Activate the measurements when we have reached 'minRedundancy' members.
     * Note that only one member (member number 'minRedundancy') will actually
     * be performing invocations.  The other members will only respond to
     * invocations.
     */
    if (view.memberHasPosition(minRedundancy-1, myId)) {
      rts.activate();
    }

    /*
     * Activate the measurements when we have reached 'minRedundancy' members.
     * In this scenario all members will start to perform invocations, when
     * the total number of group members reach 'minRedundancy'.  However, they
     * will also all be responding to invocations from all the other members.
     * Thus, response time deteriorates rapidly with growing number of members.
     */
//    if (view.size() >= minRedundancy) {
//      rts.activate();
//    }
  }

  public void hasLeft() { }

  public void prepareChange() {}


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Main method
  ////////////////////////////////////////////////////////////////////////////////////////////

  public static void main(String[] argv)
    throws Exception
  {
    RemoteTestServer server = new RemoteTestServer();
  }

} // END RemoteTestServer
