/*
 * Copyright (c) 1998-2004 The Jgroup Team.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package jgroup.test.hello;

import java.rmi.AccessException;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;

import jgroup.core.GroupManager;
import jgroup.core.JgroupException;
import jgroup.core.MembershipListener;
import jgroup.core.View;
import jgroup.core.registry.LookupService;

/**
 *  Group client program for Hello
 *
 *  @author Hein Meling
 *  @since Jgroup 2.2
 */
public class HelloGroupClient
  implements MembershipListener
{

  public HelloGroupClient()
    throws JgroupException, AccessException, RemoteException, NotBoundException
  {
    GroupManager gm = GroupManager.getGroupManager(this);
    LookupService registry = (LookupService) gm.getService(LookupService.class);
    /*
     * Retrieve a proxy for an object group implementing the Hello
     * interface registered under the name "Jgroup/HelloServer".
     */
    Hello server = (Hello) registry.lookup("Jgroup/HelloServer");
    /*
     * Invoking method sayHello, which returns an object of type Answer
     * containing a string produced by the contacted object plus
     * the time at which the groups members installed the last view.
     */
    for (int i = 0; i < 20; i++) {
      Answer answer = server.sayHello();
      System.out.println(answer);
    }
  }


  public static void main(String argv[])
    throws Exception
  {
    new HelloGroupClient();
    /*
     * There may be several threads blocking (e.g., the threads pooling
     * for registry instances), thus we just exit everything.
     */
    System.exit(0);
  }


  public void viewChange(View view)
  {
    System.out.println(view);
  }

  public void prepareChange() { }

  public void hasLeft() { }

} // END HelloGroupClient
