/*
 * Copyright (c) 1998-2002 The Jgroup Team.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package jgroup.test.jini;

import java.rmi.RemoteException;

import jgroup.core.GroupManager;
import jgroup.core.MemberId;
import jgroup.core.MembershipListener;
import jgroup.core.View;
import jgroup.relacs.gm.TimestampService;
import jgroup.util.Network;

import org.apache.log4j.Logger;

import com.sun.jini.start.LifeCycle;


/**
 * Simple Hello server aimed at demonstrating the capabilities of
 * Greg.
 *
 * @author  Alberto Montresor
 * @since   Jgroup 1.0
 */
public class HelloServer
  implements Hello, MembershipListener
{
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Logger
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** Obtain logger for this class */
  private static final Logger log = Logger.getLogger(HelloServer.class);


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Fields
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** Message returned by sayHello */
  private String answer;

  /** Service proxies */
  private TimestampService timestampService;
  

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constructor and main
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   * Constructor compatible with the jini service starter. 
   *
   * @param configArgs <code>String</code> array whose elements are
   *                   the arguments to use when creating the server.
   * @param lifeCycle  instance of <code>LifeCycle</code> that, if
   *                   non-<code>null</code>, will cause this object's
   *                   <code>unregister</code> method to be invoked during
   *                   shutdown to notify the service starter framework that
   *                   the reference to this service's implementation can be
   *                   'released' for garbage collection. A value of
   *                   <code>null</code> for this argument is allowed.
   *
   * @throws Exception If there was a problem initializing the service.
   */
  public HelloServer(String[] configArgs, LifeCycle lifeCycle) 
    throws Exception
  {
    /*
     * Prepares answer message
     */
    answer = new String("Hello from " + Network.getLocalHostName());

    /*
     * Obtains the group manager
     */
    GroupManager gm = GroupManager.getGroupManager(this);

    /*
     * Obtains the service proxies
     */
//    timestampService = (TimestampService) gm.getService(TimestampService.class);
  }


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods from Hello
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  External method
   */
   public String sayHello()
    throws RemoteException
  {
    log.debug("sayHello: returning " + answer);
    return answer;
  }


  /**
   * Returns the global timestamp
   */
  public long getGlobalTimestamp() 
    throws RemoteException
  {
    return 0;
//    return timestampService.getTimestampCounter();   
  }


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods from MembershipListener
  ////////////////////////////////////////////////////////////////////////////////////////////

  public void viewChange(View v)
  {
    log.debug("View id: " + v.getVid());
    MemberId[] members = v.getMembers();
    for (int i=0; i < members.length; i++)
      log.debug("Member " + i + ": " + members[i]);
  }

  public void prepareChange()
  {
  }

  public void hasLeft()
  {
  }
} // END HelloServer

