/*
 * Copyright (c) 1998-2005 The Jgroup Team.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */
package jgroup.test.jini.txn;

import java.rmi.RemoteException;

import jgroup.core.ExternalGMIListener;


/**
 * Remote interface for the bank services.
 * 
 * @author Rohnny Moland
 */
public interface Bank 
  extends ExternalGMIListener
{
  /**
   * Get balance of an account.
   * 
   * @param accNumber Account number
   * @return the balance of the account
   * @throws RemoteException
   */
  Integer getBalance(long accNumber)
    throws RemoteException;

  /**
   * Do a withdraw operation on given account.
   * 
   * @param accountNumber
   *   The account number to use
   * @param amount
   *   The amount to withdraw
   * @param transactionID
   *   The transaction id to use
   * @throws RemoteException
   */
  void withdraw(long accountNumber, int amount, long transactionID)
    throws RemoteException;
   
  /**
   * Do a deposit operation on given account.
   * 
   * @param accountNumber
   *   The account number to use
   * @param amount
   *   The amount to deposit
   * @param txn
   *   The transaction id to use
   * @throws RemoteException
   */
  void deposit(long accountNumber, int amount, long transactionID)
    throws RemoteException;
  
  /**
   * Get the name of a bank service.
   * 
   * @return Name of bank
   * @throws RemoteException
   */
  public String getName()
    throws RemoteException;

}
