/*
 * Copyright (c) 1998-2002 The Jgroup Team.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package jgroup.test.multicast;

import java.io.IOException;
import java.io.InputStream;

import jgroup.core.GroupManager;
import jgroup.core.JgroupException;
import jgroup.core.MemberId;
import jgroup.core.MembershipListener;
import jgroup.core.MembershipService;
import jgroup.core.View;
import jgroup.core.multicast.MulticastListener;
import jgroup.core.multicast.MulticastService;

/**
 * This is a very simple test showing the multicast capabilities
 * of Jgroup (not RMI-based).
 *
 * @author  Alberto Montresor
 * @since   Jgroup 1.0
 */
public class MulticastObjectServer
implements MembershipListener, MulticastListener
{

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Static fields
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** Protocol name used to distinguish messages from other protocols. */
  private static final String PROTOCOL_NAME = "Test";


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Fields
  ////////////////////////////////////////////////////////////////////////////////////////////

  MembershipService   membershipService;
  MulticastService    multicastService;
  MemberId            me;

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constructor
  ////////////////////////////////////////////////////////////////////////////////////////////

  MulticastObjectServer()
    throws JgroupException, IOException
  {
    GroupManager gm = GroupManager.getGroupManager(this);
    membershipService = (MembershipService) gm.getService(MembershipService.class);
    multicastService = (MulticastService) gm.getService(MulticastService.class);
    membershipService.join(7);
    me = membershipService.getMyIdentifier();
  }


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Main method
  ////////////////////////////////////////////////////////////////////////////////////////////

  public static void main(String[] argv)
    throws Exception
  {
    MulticastObjectServer t = new MulticastObjectServer();
  }

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Upcalls
  ////////////////////////////////////////////////////////////////////////////////////////////

  public void viewChange(View view)
  {
    MemberId[] members = view.getMembers();

    System.out.println("New view identifier: " + view.getVid());
    for (int i=0; i < members.length; i++)
      System.out.println(members[i]);

    try {
      if (me != null) {
        System.out.println("Send msg: " + me);
        multicastService.mcast(PROTOCOL_NAME, me, null);
      }
    } catch (Exception e) {
      System.out.println("Mcast exception: " + e);
      e.printStackTrace();
    }
  }

  /**
   *  Returns a string naming the protocol implemented by this multicast
   *  listener.
   */
  public String getProtocolName() 
  {
    return PROTOCOL_NAME;
  }

  public Object deliverStream(InputStream stream, MemberId sender, int seqNo)
  {
    System.out.print("MulticastObjectServer: deliverStream: ");
    return null;
  }


  public Object deliverObject(Object obj, MemberId sender, int seqNo)
  {
    System.out.println("MulticastObjectServer: deliverObject: " + obj);
    return null;
  }


  public void prepareChange()
  {
  }


  public void hasLeft()
  {
  }

} // END MulticastObjectServer

