/*
 * Copyright (c) 1998-2004 The Jgroup Team.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package jgroup.test.multicast;

import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.OutputStream;
import java.rmi.RemoteException;

import jgroup.core.GroupManager;
import jgroup.core.JgroupException;
import jgroup.core.MemberId;
import jgroup.core.MembershipListener;
import jgroup.core.View;
import jgroup.core.multicast.AckListener;
import jgroup.core.multicast.MulticastListener;
import jgroup.core.multicast.MulticastService;
import jgroup.relacs.gm.DispatcherService;

/**
 * Simple multicast/total order test server.
 *
 * @author Hein Meling
 * @since Jgroup 2.2
 */
public class TotalOrderTestSend
  implements MulticastListener, MembershipListener
{

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Static fields
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** Protocol name used to distinguish messages from other protocols. */
  private static final String PROTOCOL_NAME = "Total2";


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Fields
  ////////////////////////////////////////////////////////////////////////////////////////////

  private volatile boolean waitForView = false;


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constructor
  ////////////////////////////////////////////////////////////////////////////////////////////

  private TotalOrderTestSend()
    throws JgroupException, IOException
  {
    GroupManager gm = GroupManager.getGroupManager(this);
    final DispatcherService dispatcher =
      (DispatcherService) gm.getService(DispatcherService.class);
    final MulticastService multicastService =
      (MulticastService) gm.getService(MulticastService.class);
    Thread t = new Thread() {
      public void run() {
        byte[] b = new byte[100];
        boolean doMcast;
        while (true) {
          doMcast = true;
          System.out.println("Here");
          try {
            OutputStream bout = multicastService.getMessage(PROTOCOL_NAME);
            ObjectOutputStream out = new ObjectOutputStream(bout);
            out.writeUTF("Hei du!");
            out.close();
//            SynchAckListener ackListener = new SynchAckListener(dispatcher);
            GroupAckListener ackListener = new GroupAckListener();
            while (!waitForView && doMcast) {
              System.out.println("cnt=" + cnt++);
              multicastService.mcast(bout, ackListener);
              doMcast = false;
            }
            Thread.sleep(200);
            System.out.println(ackListener.getResult());
//            Object[] results = (Object[]) ackListener.getResults();
//            for (int i = 0; i < results.length; i++) {
//              System.out.println("i=" + i + ": " + results[i]);
//            }
          } catch (Exception e) {
            e.printStackTrace();
          }
        }
      }
    };
    t.start();  
  }

int cnt = 0;
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Main method
  ////////////////////////////////////////////////////////////////////////////////////////////

  public static void main(String[] argv)
    throws Exception
  {
    TotalOrderTestSend tots = new TotalOrderTestSend();
  }

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods from MembershipListener
  ////////////////////////////////////////////////////////////////////////////////////////////

  public void viewChange(View view)
  {
    System.out.println("View: " + view);
    waitForView = false;
  }

  public void prepareChange()
  {
    waitForView = true;
  }

  public void hasLeft() { }


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods from MulticastListener
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Returns a string naming the protocol implemented by this multicast
   *  listener.
   */
  public String getProtocolName() 
  {
    return PROTOCOL_NAME;
  }

  public Object deliverStream(InputStream stream, MemberId sender, int seqNo)
  {
    try {
      ObjectInputStream ios = new ObjectInputStream(stream);
      String s = ios.readUTF();
//      System.out.println("TotalOrderTestSend: deliverStream: " + s);
    } catch (IOException e) {
      e.printStackTrace();
    }
    return "Hei du og! " + sender + ": " + seqNo;
  }


  public Object deliverObject(Object obj, MemberId sender, int seqNo)
  {
    System.out.println("MulticastTestServer: deliverObject: " + obj);
    return null;
  }

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Nested class implementing the AckListener interface
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** Handles multicast return messages */
  private class GroupAckListener
    implements AckListener
  {

    /** States */
    private static final int STARTING  = 1;
    private static final int WAITING   = 2;
    private static final int COMPLETED = 3;

    /** Current state */
    private int state = STARTING;

    /** Number of unacknowledge messages */
    private int missing;

    /** Results from servers */
    private Object[] results;

    /** Flag completed servers */
    private boolean[] completed;

    /** Update internal state */
    private synchronized void updateState()
    {
      if (missing==0) {
        state = COMPLETED;
        notifyAll();
      }
    }

    /* (non-javadoc)
     * @see jgroup.core.multicast.AckListener#ack(MemberId, int, Object)
     */
    public void ack(MemberId id, int pos, Object obj)
      throws RemoteException 
    {
      System.out.println("Recieved ACK from "+id+", pos="+pos+", with reply " + obj);

      results[pos] = obj;
      completed[pos] = true;
      missing--;
      updateState();
    }

    /* (non-javadoc)
     * @see jgroup.core.multicast.AckListener#notifyView(View)
     */
    public void notifyView(View view)
      throws RemoteException
    {
      missing = view.size();
      results = new Object[missing];
      completed = new boolean[missing];
      state = WAITING;
    }

    /* (non-javadoc)
     * @see jgroup.core.multicast.AckListener#viewChange()
     */
    public void viewChange()
      throws RemoteException
    {
      missing = 0;
      updateState();
    }
    
    /** Retrieves the invocation result */
    public synchronized Object getResult()
    {
      while (state != COMPLETED) {
        try { wait(); } catch (InterruptedException iex) {}
      }

      // Select first result
      for (int i = 0; i < results.length; ++i)
        if (completed[i])
          return results[i];

      // No results returned
      System.out.println("No results!");
      return null;      
    }

  } // END GroupAckListener

} // END MulticastObjectServer
