/*
 * Copyright (c) 1998-2002 The Jgroup Team.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package jgroup.test.performance;

import java.rmi.RemoteException;

import jgroup.core.GroupManager;
import jgroup.core.JgroupException;
import jgroup.core.MembershipListener;
import jgroup.core.View;
import jgroup.core.protocols.Anycast;
import jgroup.core.protocols.Multicast;
import jgroup.util.log.Eventlogger;

/**
 *  Jgroup server for measuring the performance of GMI invocations.
 *
 *  @author Alberto Montresor
 *  @author Hein Meling
 *  @since Jgroup 1.2
 */
public class PerformanceServer
  implements SpeedTest, RemoteShutdown, MembershipListener
{

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Fields (non-shared state)
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** Counter for tracking the number of invocations performed */
  private int cnt = 0;


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constructor
  ////////////////////////////////////////////////////////////////////////////////////////////

  public PerformanceServer()
    throws JgroupException
  {
    GroupManager gm = GroupManager.getGroupManager(this);
    System.out.println("PerformanceServer initialized");
  }


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods from SpeedTest
  ////////////////////////////////////////////////////////////////////////////////////////////

  public void mtest()
    throws RemoteException
  {
    System.out.print("+");
    cnt++;
    if ((cnt%20)==0) {
      System.out.println(": " + cnt);
    }
  }

  public byte[] mtest(byte[] x)
    throws RemoteException
  {
    System.out.print("+");
    cnt++;
    if ((cnt%20)==0) {
      System.out.println(": " + cnt);
    }
    return x;
  }

  public void vmtest(byte[] x)
    throws RemoteException
  {
    return;
  }

  public String mtest(String x)
    throws RemoteException
  {
    return x;
  }

  public void test()
    throws RemoteException
  {
  }

  public byte[] test(byte[] x)
    throws RemoteException
  {
    return x;
  }

  public String test(String x)
    throws RemoteException
  {
    return x;
  }


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods from RemoteShutdown
  ////////////////////////////////////////////////////////////////////////////////////////////

  @Multicast public void preShutdown()
    throws RemoteException
  {
    if (Eventlogger.ENABLED) {
      Eventlogger.logEventFlush("PreShutdown");
    }
  }

  @Anycast public void shutdown(final int delay)
    throws RemoteException
  {
    new Thread() {
      public void run() {
        System.out.println("I'm shuting down in " + delay + " milliseconds.");
        if (delay > 0) {
          try { sleep(delay); } catch (InterruptedException e) { }
        }
        System.exit(0);
      }
    }.start();
  }


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods from MembershipListener
  ////////////////////////////////////////////////////////////////////////////////////////////

  public void viewChange(View view)
  {
    System.out.println(view);
  }


  public void hasLeft() {  }


  public void prepareChange() {  }


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Main method
  ////////////////////////////////////////////////////////////////////////////////////////////

  public static void main(String[] argv)
    throws Exception
  {
    new PerformanceServer();
  }

} // END PerformanceServer
