/*
 * Copyright (c) 1998-2002 The Jgroup Team.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package jgroup.test.performance;

import java.rmi.RemoteException;

import jgroup.core.GroupManager;
import jgroup.core.IID;
import jgroup.core.MemberId;
import jgroup.core.MembershipListener;
import jgroup.core.MembershipService;
import jgroup.core.View;
import jgroup.core.registry.RegistryService;
import jgroup.util.Abort;

/**
 *  Jgroup server for measuring the performance of GMI invocations.
 *
 *  @author Alberto Montresor
 *  @since Jgroup 1.1
 */
public class SpeedTestServer
  implements SpeedTest, MembershipListener
{

  private static int counter = 0;
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Main
  ////////////////////////////////////////////////////////////////////////////////////////////

  public static void main(String[] argv)
    throws Exception
  {
    SpeedTestServer server = new SpeedTestServer();
    server.start();
  }

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Fields
  ////////////////////////////////////////////////////////////////////////////////////////////

  private MembershipService membershipService;
  private IID registration;

  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constructor
  ////////////////////////////////////////////////////////////////////////////////////////////

  public SpeedTestServer()
  {
  }

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods from Server
  ////////////////////////////////////////////////////////////////////////////////////////////


  public void start()
    throws Exception
  {
    GroupManager gm = GroupManager.getGroupManager(this);
    membershipService = (MembershipService) gm.getService(MembershipService.class);
    RegistryService registryService = (RegistryService) gm.getService(RegistryService.class);
    membershipService.join(101);
    registration = registryService.bind("Jgroup/SpeedTest", this);
    System.out.println("SpeedTestServer registered");
  }




  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods from SpeedTest
  ////////////////////////////////////////////////////////////////////////////////////////////

  public void mtest()
    throws RemoteException
  {
    counter++;
    System.out.print("+");
    if (counter % 20 == 0)
      System.out.println("");
  }

  public byte[] mtest(byte[] x)
    throws RemoteException
  {
    counter++;
    if (counter % 20 == 0)
      System.out.println(counter);
    return x;
  }

  public void vmtest(byte[] x)
    throws RemoteException
  {
    counter++;
    System.out.print("*");
    if (counter % 20 == 0)
      System.out.println("");
  }

  public String mtest(String x)
    throws RemoteException
  {
    counter++;
    System.out.print("+");
    if (counter % 20 == 0)
      System.out.println("");
    return x;
  }

  public void test()
    throws RemoteException
  {
    counter++;
    System.out.print("+");
    if (counter % 20 == 0)
      System.out.println("");
  }

  public byte[] test(byte[] x)
    throws RemoteException
  {
    counter++;
    System.out.print("+");
    if (counter % 20 == 0)
      System.out.println("");
    return x;
  }

  public String test(String x)
    throws RemoteException
  {
    counter++;
    System.out.print("+");
    if (counter % 20 == 0)
      System.out.println("");
    return x;
  }

  /**
  * Shut down the server
   */
  public void shutdown()
    throws RemoteException
  {
    new Thread() {
      public void run() {
        try {
          sleep(3000);
        } catch (InterruptedException e) {
        }
        Abort.exit(0);
      }
    }.start();
  }

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods from MembershipListener
  ////////////////////////////////////////////////////////////////////////////////////////////

  public void viewChange(View v)
  {
    System.out.println("- New View ------------------------");
    MemberId[] adds = v.getMembers();
    for (int i=0; i < adds.length ; i++)
      System.out.println(adds[i]);
    System.out.println("-----------------------------------");
  }

  public void hasLeft()
  {
  }

  public void prepareChange()
  {
  }

} // END SpeedTestServer

