/*
 * Copyright (c) 1998-2002 The Jgroup Team.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package jgroup.test.performance.arm;

import java.rmi.RemoteException;

import jgroup.core.ConfigManager;
import jgroup.core.GroupManager;
import jgroup.core.JgroupException;
import jgroup.core.protocols.Anycast;
import jgroup.core.protocols.Multicast;
import jgroup.test.performance.RemoteShutdown;
import jgroup.test.performance.SpeedTest;
import jgroup.util.Network;
import jgroup.util.log.Eventlogger;
import jgroup.util.log.ReplicaEvent;
import static jgroup.util.log.ReplicaEvent.Type.Initialized;

/**
 *  Jgroup server for measuring the performance of GMI invocations and
 *  server recovery time.
 *
 *  @author Hein Meling
 *  @since Jgroup 1.2
 */
public class ReplicatedServer
  implements SpeedTest, RemoteShutdown
{

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Fields (non-shared state)
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** The local host name of the server replying to client requests. */
  private static final String localHost = Network.getLocalMachineName();


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constructor
  ////////////////////////////////////////////////////////////////////////////////////////////

  public ReplicatedServer()
    throws JgroupException
  {
    ConfigManager.init();
    GroupManager gm = GroupManager.getGroupManager(this);
    if (Eventlogger.ENABLED)
      Eventlogger.logEventFlush(new ReplicaEvent(Initialized, gm.getGroupId()));
    System.out.println("ReplicatedServer initialized");
  }


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods from SpeedTest
  ////////////////////////////////////////////////////////////////////////////////////////////

  @Multicast public void mtest()
    throws RemoteException
  {
  }

  private int cnt = 0;


  @Multicast public byte[] mtest(byte[] x)
    throws RemoteException
  {
//    System.out.print("+");
//    if ((cnt % 20) == 0) {
//      System.out.println(cnt);
//    }
//    cnt++;
    return x;
  }

  @Multicast public void vmtest(byte[] x)
    throws RemoteException
  {
    System.out.print("+");
    if ((cnt % 20) == 0) {
      System.out.println(cnt);
    }
    cnt++;
  }

  @Multicast public String mtest(String x)
    throws RemoteException
  {
    System.out.println(x);
    return x;
  }

  @Anycast public void test()
    throws RemoteException
  {
  }

  @Anycast public byte[] test(byte[] x)
    throws RemoteException
  {
//    cnt++;
//    if (Debug.MEASURE && cnt % 5 == 0)
//      Debug.logEvent("cnt " + cnt, "");
    return x;
  }

  @Anycast public String test(String x)
    throws RemoteException
  {
    return localHost + ": " + x;
  }


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods from RemoteShutdown
  ////////////////////////////////////////////////////////////////////////////////////////////

  @Multicast public void preShutdown()
    throws RemoteException
  {
    if (Eventlogger.ENABLED) {
      Eventlogger.logEventFlush("PreShutdown localhost: " + localHost);
    }
  }

  @Anycast public void shutdown(final int delay)
    throws RemoteException
  {
    new Thread() {
      public void run() {
        System.out.println("I'm shuting down in " + delay + " milliseconds.");
        if (delay > 0) {
          try { sleep(delay); } catch (InterruptedException e) { }
        }
        System.exit(0);
      }
    }.start();
  }


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Main method
  ////////////////////////////////////////////////////////////////////////////////////////////

  public static void main(String[] argv)
    throws Exception
  {
    ReplicatedServer server = new ReplicatedServer();
  }

} // END ReplicatedServer
