/*
 * Copyright (c) 1998-2002 The Jgroup Team.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package jgroup.test.performance.arm;

import java.util.HashMap;
import java.util.Map;

import jgroup.core.GroupManager;
import jgroup.core.JgroupException;
import jgroup.core.MemberId;
import jgroup.core.MembershipListener;
import jgroup.core.MembershipService;
import jgroup.core.MergingListener;
import jgroup.core.View;

import org.apache.log4j.Logger;

/**
 *  Allocator server
 *
 *  @author Hein Meling
 *  @since Jgroup 1.2
 */
public class StatefulServer
  implements MembershipListener, MergingListener
{

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Logger
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** Obtain logger for this class */
  private static final Logger log = Logger.getLogger(StatefulServer.class);


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Fields (non-shared state)
  ////////////////////////////////////////////////////////////////////////////////////////////

  /* My MemberId */
  private MemberId p;

  /* The current view composition */
  private View viewComp;

  /* Number of resources sent */
  private Map sent = new HashMap(10);

  /* Value of <code>sent</code> at the beginning of the current view */
  private Map lastSent = new HashMap(10);

  /* Number of resources received */
  private Map<MemberId,Integer> received = new HashMap<MemberId,Integer>(10);


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constructor
  ////////////////////////////////////////////////////////////////////////////////////////////

  public StatefulServer()
    throws JgroupException
  {
    GroupManager gm = GroupManager.getGroupManager(this);
    MembershipService pgms = (MembershipService) gm.getService("PGMS");
    p = pgms.getMyIdentifier();
    Integer initialValue = new Integer(50);
    received.put(p, initialValue);
    System.out.println("StatefulServer initialized");
  }


  ////////////////////////////////////////////////////////////////////////////////////////////
  // MergingListener methods
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   * @see jgroup.core.MergingListener#getState(jgroup.core.MemberId[])
   */
  public Object getState(MemberId[] dests)
  {
    if (log.isDebugEnabled()) {
      for (int i=0; i < dests.length; i++)
        log.debug("getState dests["+i+"]: "+dests[i]);
    }
    return null;
  }


  /**
   * @see jgroup.core.MergingListener#putState(java.lang.Object, jgroup.core.MemberId[])
   */
  public void putState(Object status, MemberId[] sources)
  {
    if (log.isDebugEnabled()) {
      for (int i=0; i < sources.length; i++)
        log.debug("putState sources["+i+"]: "+sources[i]);
    }
  }


  /**
   * @see jgroup.core.MembershipListener#viewChange(jgroup.core.View)
   */
  public void viewChange(View view)
  {
    viewComp = view;
  }


  public void prepareChange() {  }

  public void hasLeft() {  }


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Main method
  ////////////////////////////////////////////////////////////////////////////////////////////

  public static void main(String[] argv)
    throws Exception
  {
    StatefulServer server = new StatefulServer();
  }

} // END StatefulServer
