/*
 * Copyright (c) 1998-2002 The Jgroup Team.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package jgroup.test.performance.upgrade;

import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.PrintStream;

import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JSlider;
import javax.swing.SwingConstants;
import javax.swing.border.TitledBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import jgroup.core.ConfigManager;
import jgroup.core.registry.DependableRegistry;
import jgroup.test.performance.SpeedTest;

import org.apache.log4j.Logger;

/**
 *  This class implements a Swing-based GUI for the UpgradeClientTest.
 *  It supports an interface that allows to control the rate at which
 *  the client sends requests to the server and one of the predefined
 *  method that is invoked on the upgrade server. 
 *
 *  @author Marcin Solarski
 *  @since Jgroup 2.1
 */

public class Generator implements ChangeListener, ActionListener {

  /** Obtain logger for this class */
  private static Logger log = Logger.getLogger(Generator.class);


  private UpgradeClientTest test = null;

  private final int TP_MAX = 1000;
  private final int TP_TICK = 250;
  private final int METHOD_MAX = 4;
  private final long RATE_UPDATE_TIME = 2000;
  private final long EXPERIMENT_GAP = 2000;
  private float avRespTime = 0;
  private int method;
  private int testTime = 5; //in s
  private int tp;
  private int rate;
  private boolean running, calibrated, exiting = false;

  private JFrame frame;
  private JButton calButton;
  private JButton startButton;
  private JRadioButton b1, b2, b3, b4;
  private JLabel rateLabel;
  private JSlider methodSlider;
  private JPanel methodPanel;
  private TitledBorder methodBorder;
  private JSlider tpSlider;
  private TitledBorder tpBorder;
  private Font bigFont;

  class RateChecker extends Thread {

    public void run() {
      setPriority(MIN_PRIORITY);
      while (running) {
	rateLabel.setText(""+avRespTime);
        try {
          Thread.sleep(RATE_UPDATE_TIME);
        }
        catch (Exception ex) {}
      }
    }
  }

  class Invocator extends Thread {
   
   DependableRegistry reg  = null;
   SpeedTest server  = null;
   PrintStream out = null;
   boolean initOK = false; 
   long ei = 0;


    public Invocator() {
    }

    private void contactServer() { 
      while (! initOK ) {
        try {
          test = new UpgradeClientTest("UpgradeServer");
          initOK = true;
        } catch(Exception e) {
	   log.warn("Cannot contact the server. Waiting 5 sec and trying again..." + e.getMessage());
	   try{ Thread.sleep(5000); } catch(InterruptedException ex) {}
	}
      }
   }


    public void run() {
      contactServer();
      calButton.setEnabled(true);
      //the test invocation handling should have the highest priority
      setPriority(MAX_PRIORITY);
      try {
        while ( true ) { 
	  if( running ) {
	     log.debug("Experiment " + ei + " with rate = " + tp + " test time = " + testTime );
	     avRespTime = test.generate_traffic(tp, testTime); 
	     rateLabel.setText(""+avRespTime);
	     ei++;
	  }
	  if( exiting ) break;
	  try {Thread.sleep(EXPERIMENT_GAP); } catch(InterruptedException e) {};
	}
      }
      catch(Exception e) {
        log.info("While sending request, an exception caught: ", e);
      }
    }
  }

  public Generator(int method, int tp) {
    this.method = method;
    this.tp = tp;


   if (this.method < 0) this.method = 0;
   if (this.method > METHOD_MAX) this.method = METHOD_MAX;
   if (this.tp < 1) this.tp = 1;
   if (this.tp > TP_MAX) this.tp = TP_MAX;

  }

  
    public void actionPerformed(ActionEvent e) {
      Object src = e.getSource(); 
      if (src == calButton) {
	    log.debug("Calibration is started");
	    rateLabel.setText(""+test.calibrate(method));
	    calibrated = true;
            startButton.setEnabled(true);
	    log.debug("Calibration done");
       } else if (src == startButton ) {
         if (running ) {
	         running = false;
	        log.debug("Experiment will be stoped");
		startButton.setText("Start");
	 } else {
	        running = true;
	        log.debug("Experiment will be started");
		startButton.setText("Stop");
         }
       } else if (src == b1) {
	       log.debug("Changing method to 1");
        } else if (src == b2) {
	       log.debug("Changing method to 2");
        } else if (src == b3) {
	       log.debug("Changing method to 3");
        } else if (src == b4) {
	       log.debug("Changing method to 4");
      }

    }


  public void stateChanged(ChangeEvent evt) {
    Object src = evt.getSource();
    //log.debug(evt);
    if (src == methodSlider) {
      method = methodSlider.getValue();
      method = (method + 1) % 4;
      methodBorder.setTitle(" Method  " + method );
    } else if (src == tpSlider) {
      tp = tpSlider.getValue();
      tpBorder.setTitle(" troughput " + tp + " rps ");
      tp++;
    } 
  }

  public void init() {

    bigFont = new Font( null, Font.BOLD, 
	 new Integer( System.getProperty("jgroup.test.font.big", "22")).intValue());
    
    frame = new JFrame("Generator" );

    GridBagLayout gridbag = new GridBagLayout();
    GridBagConstraints constr = new GridBagConstraints();
    Container cpane = frame.getContentPane();
    cpane.setLayout(gridbag);

    calButton = new JButton("Calibrate", new ImageIcon("resources/configure.jpg", "Calibrate"));
    calButton.setDefaultCapable(true); 
    calButton.addActionListener(this);
    calButton.setEnabled(false);
    constr.fill = GridBagConstraints.BOTH;
    constr.weightx = 0.5;
    constr.weighty = 0.5;
    constr.gridx = 0;
    constr.gridy = 2;
    constr.gridwidth = 1;
    gridbag.setConstraints(calButton, constr);
    cpane.add(calButton);

   startButton = new JButton("Start", new ImageIcon("resources/start.jpg", "Start"));
    startButton.setDefaultCapable(true); 
    startButton.addActionListener(this);
    startButton.setEnabled(false);
    constr.fill = GridBagConstraints.BOTH;
    constr.weightx = 0.5;
    constr.weighty = 0.5;
    constr.gridx = 1;
    constr.gridy = 2;
    constr.gridwidth = 1;
    gridbag.setConstraints(startButton, constr);
    cpane.add(startButton);

     
    rateLabel = new JLabel(rate + " ");
    rateLabel.setFont(bigFont);
    rateLabel.setForeground(Color.black);
    rateLabel.setHorizontalAlignment(SwingConstants.RIGHT);
    rateLabel.setBorder(BorderFactory.createTitledBorder(" Response Time  "));
    constr.fill = GridBagConstraints.BOTH;
    constr.weightx = 1.0;
    constr.weighty = 0.5;
    constr.gridx = 2;
    constr.gridy = 2;
    constr.gridwidth = 1;
    gridbag.setConstraints(rateLabel, constr);
    cpane.add(rateLabel);

    JLabel logo = new JLabel(new ImageIcon("resources/logo.jpg", "Logo"));
    constr.fill = GridBagConstraints.BOTH;
    constr.weightx = 1.0;
    constr.weighty = 0.5;
    constr.gridx = 4;
    constr.gridy = 2;
    constr.gridwidth = 1;
    gridbag.setConstraints(logo, constr);
    cpane.add(logo);

    methodPanel = new JPanel();
    methodPanel.setBorder( BorderFactory.createTitledBorder(" method to invoke"  ));
    ButtonGroup methodGroup = new ButtonGroup();
    b1 = new JRadioButton("test()", true);
    methodGroup.add(b1);
    methodPanel.add(b1);
    b1.addActionListener(this);
    b2 = new JRadioButton("mtest()", false);
    methodGroup.add(b2);
    methodPanel.add(b2);
    b2.addActionListener(this);
    b3 = new JRadioButton("test(byte[])", false);
    methodGroup.add(b3);
    methodPanel.add(b3);
    b3.addActionListener(this);
    b4 = new JRadioButton("mtest(byte[])", false);
    methodGroup.add(b4);
    methodPanel.add(b4);
    b4.addActionListener(this);
    constr.fill = GridBagConstraints.BOTH;
    constr.weightx = 1.0;
    constr.weighty = 0.5;
    constr.gridx = 0;
    constr.gridy = 0;
    constr.gridwidth = 3;
    gridbag.setConstraints(methodPanel, constr);
    cpane.add(methodPanel);


   tpBorder =
      BorderFactory.createTitledBorder(" throughput " + tp + " rps ");

    tpSlider = new JSlider(SwingConstants.HORIZONTAL, 0, TP_MAX, tp);
    tpSlider.setBorder(tpBorder);
    tpSlider.setMajorTickSpacing(TP_TICK);
    tpSlider.setMinorTickSpacing(TP_TICK / 2);
    tpSlider.setPaintTicks(true);
    tpSlider.setPaintLabels(true);
    tpSlider.addChangeListener(this);
    constr.fill = GridBagConstraints.BOTH;
    constr.weightx = 1.0;
    constr.weighty = 0.5;
    constr.gridx = 0;
    constr.gridy = 1;
    constr.gridwidth = 3;
    gridbag.setConstraints(tpSlider, constr);
    cpane.add(tpSlider);

    
    frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    frame.setSize(400, 220);
    Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
    frame.setLocation(
      (d.width - frame.getSize().width) / 2,
      (d.height - frame.getSize().height) / 2);
    frame.setVisible(true);

    calibrated = false;
    running = false;

    //RateChecker checker = new RateChecker();
    //checker.start();
    Invocator invocator = new Invocator();
    invocator.start();
  }

  public static void main(String args[])
  {
    int method = 0;
    int tp = 1;
    try {
      ConfigManager.init();
      Generator gen = new Generator(method, tp);
      gen.init();
    } catch (Exception ex) {
      ex.printStackTrace();
    }
  }
}
