/*
 * Copyright (c) 1998-2002 The Jgroup Team.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */
 
package jgroup.arm.recovery;

import jgroup.relacs.config.AppConfig;

import org.apache.log4j.Logger;


/**
 *  If the current number of replicas drops below the initial redundancy
 *  specified for this replica, and the number of available hosts in
 *  this partition is greater than the initial redundancy then create
 *  new replicas to compensate for the lost redundancy.
 *
 *  @author Hein Meling
 *  @since Jgroup 1.2
 */
public class KeepInitialInPartition
  extends KeepMinimalInPartition
{

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Logger
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** Obtain logger for this class */
  private static final Logger log = Logger.getLogger(KeepInitialInPartition.class);


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Helper methods
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Returns the number of replicas that needs to be recovered to
   *  maintain the initial redundancy level specified.  If no recovery
   *  is needed, zero is returned.
   */
  protected int deltaRedundancy(AppConfig app)
  {
    int delta = app.getInitialRedundancy() - app.getCurrentGroupSize();
    return (delta >= 0) ? delta : 0;
  }

} // END KeepInitialInPartition
