/*
 * Copyright (c) 1998-2002 The Jgroup Team.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package jgroup.core;

import java.io.Externalizable;
import java.net.InetAddress;

/**
 *  Interface <code>EndPoint</code> represents a communication endpoint.
 *  It is composed of an IP address together with a port number.
 *  Objects implementing this interface are used to identify JVM
 *  machines hosting Jgroup applications.
 *
 *  @author Alberto Montresor
 *  @author Hein Meling
 *  @since Jgroup 1.2
 */
public interface EndPoint
  extends Externalizable, Comparable
{

  /**
   *  Returns the IP address contained in this endpoint
   */
  public InetAddress getAddress();

  /**
   *  Returns the IP address contained in this endpoint, enclosed
   *  in an integer value.
   */
  public int getIntAddress();

  /**
   *  Returns the port number contained in this endpoint.
   */
  public int getPort();

  /**
   *  Returns true if this endpoint is the local endpoint.
   */
  public boolean isLocal();

  /**
   *  Return true if this endpoint is a multicast endpoint.
   */
  public boolean isMulticastEndPoint();

} // END EndPoint
