/*
 * Copyright (c) 1998-2002 The Jgroup Team.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package jgroup.core;

import java.rmi.Remote;

/**
 *  In Jgroup, an <i>external remote interface</i> is an interface that
 *  declare a set of methods implemented by an object group that may be
 *  invoked from a remote Java virtual machine.  An external remote
 *  interface must satisfy the following conditions:
 *
 *  <UL>
 *  <LI>It must at least extend, either directly or indirectly, the
 *  interface <code>ExternalGMIListener</code>.  </LI>
 *  <LI>Each remote method declaration in the external remote interface
 *  must include the exception <code>java.rmi.RemoteException</code> in
 *  its <i>throws</i> clause, in addition to any application-specific
 *  exceptions (note that application-specific exceptions do not have to
 *  extend <code>java.rmi.RemoteException</code>).  </LI>
 *  </UL>
 *
 *  The <code>ExternalGMIListener</code> is a marker interface that
 *  declares no methods.
 *
 *  @author Alberto Montresor
 *  @since Jgroup 0.9
 */
public interface ExternalGMIListener
  extends Remote
{
}
