/*
 * Copyright (c) 1998-2002 The Jgroup Team.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package jgroup.core;


/**
 *  This interface must be implemented by all layers that should be
 *  constructable by runtime parsing of the XML layer composition
 *  framework.  Typically, a service interface (e.g., MembershipService)
 *  will extend this interface to avoid additional interface
 *  declarations in the actual layer implementation.
 *
 *  @author Hein Meling
 *  @since Jgroup 1.2
 */
public interface Layer
{

  /**
   *  This is a special interface implemented by layers that has to perform
   *  post construction finalization.
   *
   *  @author Hein Meling
   *  @since Jgroup 2.1
   */
  public interface FinalizeLayer
  {

    /**
     *  This method is invoked on the layer, once the whole Jgroup group
     *  manager construction has been completed.  The <code>complete()</code>
     *  method is supposed to implemented stuff that depends on several
     *  other group manager components having been constructed.
     * 
     *  @param server
     *    The server using a layer implementing the <code>FinalizeLayer</code>
     *    interface.
     *  @exception JgroupException
     *    Thrown if the layer could not be completed.
     */
    public void complete(Object server)
      throws JgroupException;

  } // END FinalizeLayer


  /**
   *  Add a listener to this layer.  Many layers require to be notified
   *  of events produced by other layers in order to provide its own
   *  functionallity, and this method must be invoked to notify this
   *  layer of an interested listener.  A listener may be another layer
   *  or an application server replica.
   *
   *  @param listener
   *    An <code>Object</code> representation of the listener interface.
   *    This will typically be cast to the appropriate listener interface
   *    type after checking the type with the <tt>instanceof</tt> operator.
   *  @exception IllegalArgumentException
   *    Raised if the given listener object do not implement any of the
   *    listeners provided by this layer.
   */
  public void addListener(Object listener);

} // END Layer
