/*
 * Copyright (c) 1998-2002 The Jgroup Team.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package jgroup.relacs.config;

import jgroup.core.ConfigurationException;


/**
 *  Class for holding bootstrap registry information.
 *
 *  @author Hein Meling
 *  @author Alberto Montresor
 *  @since Jgroup 1.1
 */
public class BootstrapRegistryConfig 
  implements ConfigurationObject
{

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Tags
  ////////////////////////////////////////////////////////////////////////////////////////////
  
  private static final String PORT_ATTRIB = "port";


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Default values
  ////////////////////////////////////////////////////////////////////////////////////////////

  private static final int BOOTSTRAP_REGISTRY_PORT = 5671;


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Fields
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** Registry port */
  private int port;


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constructors
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Builds a new BootstrapRegistryConfig object, using default values.
   */
  public BootstrapRegistryConfig() 
  {
    this(BOOTSTRAP_REGISTRY_PORT);
  }


  /**
   *  Builds a new BootstrapRegistryConfig object, using the specified port.
   *  
   *  @param port 
   *    bootstrap registry port
   */
  public BootstrapRegistryConfig(int port)
  {
    this.port = port;
  }

  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods from ConfigurationObject
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Parses the specified element.
   */
  public void parse(org.w3c.dom.Element elm)
    throws ConfigurationException
  {
    port = ConfigParser.getIntAttrib(elm, PORT_ATTRIB, false);
  }


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Getter methods
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Returns the port number for the bootstrap registry service.
   */
  public int getPort()
  {
    return port;
  }


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Override methods in Object
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Returns a string representation of this object
   */
  public String toString()
  {
    return "[BootstrapRegistryConfig: port = " + port +"]";
  }

} // END BootstrapRegistryConfig
