/*
 * Copyright (c) 1998-2002 The Jgroup Team.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package jgroup.relacs.events;

/**
 *  The <code>EventTags</code> interface defines symbolic constants for
 *  the tags used to distinguish events.  Events are used for
 *  communication between the daemon and group members.
 *
 *  @author Alberto Montresor
 *  @since  Jgroup 1.2
 */
public interface EventTags
{

  /**
   *  This tag identifies the events generated when a daemon delivers
   *  a message to a member.
   */
  public static final byte DELIVERY_EVENT = 1;

  /** 
   *  This tag identifies an event generated by a member to acknowledge
   *  the delivery of a message.
   */
  public static final byte DELIVERY_ACK = 2;

  /**
   *  This tag identifies the events generated when a daemon installs
   *  a view to a member.
   */
  public static final byte INSTALL_EVENT = 3;

  /** 
   *  This tag identifies an event generated by a member to acknowledge
   *  the installation of a view.
   */
  public static final byte INSTALL_ACK = 4;

  /** 
   *  This tag identifies an event generated when a daemon is about to
   *  install a new view to a member.
   */
  public static final byte PREPARE_EVENT = 5;

  /** 
   *  This tag identifies the events generated by a member to acknowledge
   *  the delivery of a prepare event.
   */
  public static final byte PREPARE_ACK= 6;

  /** 
   *  This tag identifies an event generated by a member when
   *  multicasting a message.
   */
  public static final byte MULTICAST_REQUEST = 7;

  /** 
   *  This tag identifies the events generated when a member requests
   *  to join a group.
   */
  public static final byte JOIN_REQUEST = 8;

  /** 
   *  This tag identifies the events generated when a member requests to
   *  leave a group.
   */
  public static final byte LEAVE_REQUEST = 9;

  /** 
   *  This tag identifies the events generated by a daemon to acknowledge
   *  that the member has left the group.
   */
  public static final byte MEMBER_LEFT_EVENT = 10;

  /** 
   *  This tag identifies the events generated by the dispatcher to
   *  indicate the failure to the daemon.
   */
  public static final byte DAEMON_FAILURE_EVENT = 11;

} // END EventTags
