/*
 * Copyright (c) 1998-2002 The Jgroup Team.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package jgroup.relacs.gm;

import jgroup.core.JgroupException;
import jgroup.core.Layer;

/**
 *  Service interface for the dispatcher layer. It provides methods
 *  to add the start and stop the dispatcher, and to force the
 *  dispatching of a single event.
 * 
 *  @author Alberto Montresor
 *  @author Hein Meling
 *  @since 0.8
 */
public interface DispatcherService
  extends Layer
{

  /**
   *  Start dispatching events for the given group.
   */
  public void start(int gid);


  /**
   *  Stop dispatching events
   */
  public void halt();


  /**
   *  Dispatch a single event (can be called by an upcall method)
   */
  public void dispatch(Object obj);


  /**
   *  Returns the remote dispatcher reference.
   *
   *  @throws JgroupException
   *    Raised if the dispatcher service could be not export
   *    for remote access by the Jgroup daemon.
   */
  public RemoteDispatcher getRemoteDispatcher()
    throws JgroupException;

} // END DispatcherService
