/*
 * Copyright (c) 1998-2002 The Jgroup Team.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package jgroup.relacs.gm;

import jgroup.core.IID;

/**
 *  The <code>LeaseInfo</code> class is a datastructure used by the
 *  dependable registry to support leasing.
 *
 *  @author Henning Hommeland
 *  @author Jo Andreas Lind
 *  @since Jgroup 2.1
 */
public class LeaseInfo
{
  //////////////////////////////////////////////////////////////////////////////////////////
  // Private fields
  //////////////////////////////////////////////////////////////////////////////////////////

  /* A table of IIDs */
  private IID iid[];
  
  /* A table of timestamps */
  private long timestamp[];
  
  /* Size of tables */
  private int size;
  
  /*  */
  private long leasetime;


  //////////////////////////////////////////////////////////////////////////////////////////
  // Constructor
  //////////////////////////////////////////////////////////////////////////////////////////	
  
  public LeaseInfo(int size)
  {
	iid = new IID[size];
	timestamp = new long[size];
	this.size = size;
  }
  
  
  //////////////////////////////////////////////////////////////////////////////////////////
  // Class methods
  //////////////////////////////////////////////////////////////////////////////////////////
	
  public void setTimeStamp(long ts, int place)
  {
	this.timestamp[place] = ts;
  }

  public void setIID(IID iid, int place)
  {
    this.iid[place] = iid;
  }

  public IID getIID(int place)
  {
	return iid[place];
  }
	
  public int size()
  {
	return size;
  }
	
  public long getLeasetime()
  {
	return leasetime;
  }
	
  public void setLeasetime(long rr)
  {
	this.leasetime = rr;
  }


  /**
   *  Returns true if the lease has expired for the member given by
   *  position argument <code>j</code>.  Otherwise, false is returned.
   */
  public boolean leaseExpired(int j)
  {
    return (System.currentTimeMillis() - timestamp[j]) > leasetime;
  }

} // End LeaseInfo
	