/*
 * Copyright (c) 1998-2006 The Jgroup Team.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package jgroup.relacs.gmi.protocols;

import java.io.IOException;
import java.io.InputStream;

import jgroup.relacs.gmi.InvocationResult;

/**
 * ProtocolDispatcher must be implemented by GMI protocols.
 *
 * @author Hein Meling <hein.meling@uis.no>
 */
public interface ProtocolDispatcher
{

  /**
   * Invoked to dispatch a method invocation on the local endpoint or on
   * multiple external endpoints.  The method may block until a result can
   * be returned to the caller.  It may also return <code>null</code> if
   * problems occur, e.g. connectivity problems. 
   * 
   * @param in the method to invoke, encoded in the provided <code>InputStream</code>. 
   * @return the result of the method invocation.
   * @throws IOException Raised if the method invocation read from the
   *   <code>InputStream</code> could not be executed for some reason.
   */
  public InvocationResult dispatch(InputStream in)
    throws IOException;

  /**
   * Pass the server reference to the protocol implementation.
   */
  public void setServer(Object server);

} // END ProtocolDispatcher
