/*
 * Copyright (c) 1998-2002 The Jgroup Team.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package jgroup.relacs.mss;

/*
 *  FIXME: This data structure is not dynamic and uses the index
 *  of the host in the cluster; should change to a dynamic approach.
 */
//FIXME clusterWindow is important only for members of the same
//cluster.  HEIN: What exactly does this mean?

/**
 *  The <code>ClusterWindow</code> class maintains, for each member host
 *  in a cluster, the last message that it has delivered.
 *
 *  @author Salvatore Cammarata
 *  @author Hein Meling
 *  @since Jgroup 1.2
 */
final class ClusterWindow 
  implements MssConstants
{

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Fields
  ////////////////////////////////////////////////////////////////////////////////////////////
 
  /** Last msg delivered for each member in the cluster */
  private int[] lmd;
  
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constructors
  ////////////////////////////////////////////////////////////////////////////////////////////

  ClusterWindow(int members)
  {
    lmd = new int[members];
    for (int i=0; i < members; i++)
      lmd[i] = UNDEF;
  }


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Returns the last message delivered in the cluster. 
   */
  int getLastMsgDelivered()
  {
    int min = UNDEF;
    int i=0;

    while (i < lmd.length && lmd[i] == UNDEF)
      i++;
    if (i < lmd.length)
      min = lmd[i++];
    while (i < lmd.length) {
      if (lmd[i] != UNDEF && lmd[i] < min)
        min = lmd[i];
      i++;
    }
    return min;
  }


  /**
   *  Sets the last message delivered for the host identified by the
   *  given index in this cluster.
   *  
   *  @param index
   *    The index of the host in this cluster
   *  @param lastMsgDelivered
   *    The identifier of the last message delivered by the given host
   */
  int set(int index, int lastMsgDelivered)
  {
    int ret = lmd[index];
    lmd[index] = lastMsgDelivered;
    return ret;
  }


  /**
   *  Clear the cluster window state.
   */
  void clear()
  {
    for (int i = 0; i < lmd.length; i++)
      lmd[i] = UNDEF;
  }


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Returns a string representation of this object
   */
  public String toString()
  {
    StringBuilder buf = new StringBuilder("[ClusterWindow: ");
    for (int i = 0; i < lmd.length; i++) {
      buf.append((i != 0) ? ", " : "");
      buf.append("lastMsgDlvr[");
      buf.append(i);
      buf.append("]=");
      buf.append(lmd[i]);
    }
    buf.append("]");
    return buf.toString();
  }

} // END ClusterWindow
