/*
 * Copyright (c) 1998-2002 The Jgroup Team.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package jgroup.relacs.mss;

/**
 *  The <code>MssTag</code> interface contains the tags used by the Mss.
 *
 *  @author Alberto Montresor
 *  @author Salvatore Cammarrata
 *  @author Hein Meling
 *  @since Jgroup 0.1
 */
public interface MssTag 
{

  public static final String[] TAGTYPE = new String[] {
    "INVALID TAG",
    "TOPOLOGY",
    "ROUTING",
    "FWDROUTING",
    "IAMALIVE",
    "SENT",
    "JG",
    "NACK",
    "SENTNACK",
    "REMOTENACK",
    "SYN",
    "ASYN",
    "QSYN",
    "LOCALFC",
    "EXTERNFC",
    "CONGESTION",
    "PING_OBJECT"
  };

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Topology Message Types
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** Common message tag for topology messages */
  public static final byte TOPOLOGY   = 1;

  /** The topology msg has to be broadcast when received */
  public static final byte ROUTING    = 2; 

  /** The topology msg should not be broadcast when received */
  public static final byte FWDROUTING = 3;

  /** Local topology msg */
  public static final byte IAMALIVE   = 4;

  /** The topology msg for the cluster is already sent */
  public static final byte SENT       = 5;


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Jgroup Related Message Types
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** JG message */
  public static final byte JG         = 6;
  public static final byte NOTLASTFRAGMENT = -1;


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Negative Acknowledgement Message Types
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** NACK message types */
  public static final byte NACK       = 7;
  public static final byte SENTNACK   = 8;
  public static final byte REMOTENACK = 9;


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Synchronization Message Types
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** Synchronization message */
  public static final byte SYN        = 10;

  /** Answer (reply) to a QSYN request */
  public static final byte ASYN       = 11;

  /** Query (request) for synchronization */
  public static final byte QSYN       = 12;


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Flow Control Message Types
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** FC Message Type */
  public static final byte LOCALFC    = 13;
  public static final byte EXTERNFC   = 14;


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Congestion Message Type
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** Message to indicate congestion */
  public static final byte CONGESTION = 15;

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Ping request event
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** Message to indicate congestion */
  public static final byte PING_OBJECT = 16;
  
} // END MssTag
