/*
 * Copyright (c) 1998-2002 The Jgroup Team.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package jgroup.relacs.mss;

import jgroup.core.EndPoint;
import jgroup.core.MemberId;
import jgroup.relacs.events.Event;
import jgroup.util.InMessage;

/**
 *  The <code>MssUser</code> interface should be implemented by layers
 *  using the Mss to receive callback notifications from the Mss itself.
 *
 *  @author Alberto Montresor
 *  @since  Jgroup 0.1
 */
public interface MssUser
{

  /**
   *  Invoked to notify the objects implementing this interface that
   *  a message from a remote member has been received.
   *  
   *  @param tag the tag identifying the type of message
   *  @param stream the message, represented by a <CODE>InMessage</CODE> 
   *    object
   *  @param src the sender of the message
   */
  public void remoteReceive(byte tag, InMessage data, EndPoint host);

  /**
   *  Invoked to notify the objects implementing this interface that
   *  a message from a local member has been received.
   *  
   *  @param event represents the received message
   */
  public void localReceive(Event upcall);

  /**
   *  Invoked to notify the objects implementing this interface that
   *  a variation in the perceived reachability set has been perceived.
   *
   *  @param trset   set of all (total) reachables hosts
   *  @param nrset   set of reachable hosts that were unreachable
   *  @param nuset   set of unreachable hosts that were reachable
   *  @param newinc  set of hosts who has a new incarnation identifier
   */
  public void remoteSuspect(EndPoint[] rset, EndPoint[] nrset, EndPoint[] nuset, EndPoint[] newinc);

  /**
   *  Invoked to notify the objects implementing this interface that
   *  a remote object is not responding to remote method invocations,
   *  and thus should be crashed.
   *  
   *  @param membId a callback object identifying the suspected object;
   *    this object has been stored in the Mss through the
   *    <code>addPingListener</code> method invocation on the Mss.
   */
  public void localSuspect(MemberId membId);
  
} // END MssUser
