/*
 * Copyright (c) 1998-2002 The Jgroup Team.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package jgroup.relacs.simulator;

import java.net.InetAddress;
import java.rmi.Remote;
import java.rmi.RemoteException;
import java.util.Map;

/**
 *  <code>SocketStatus</code> is a remote interface for assigning
 *  probabilities to channels in the distributed system.
 *
 *  @author Alberto Montresor
 *  @author Hein Meling
 *  @since Jgroup 0.5 (revised Jgroup 2.2)
 */
public interface SocketStatus
  extends Remote
{

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constants
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** Reliable channel */
  public static final int BASE = 100;

  /** Registry lookup name */
  public static final String SOCKET_STATUS = "SocketStatus";


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Register the given socket status object with this socket status
   *  implementation, so as to notify the other socket status instances
   *  of changes to the reachability pattern.
   */
  public void register(SocketStatus sStatus)
    throws RemoteException;

  /**
   *  This method must be invoked before the current reachability
   *  pattern will be activated.  That is, after invoking a series of
   *  <code>setStatus</code> methods, an invocation of this method
   *  will activate the newly set reachability pattern.
   */
  public void commit()
    throws RemoteException;

  /**
   *  Notifies any registered socket status implementations of the most
   *  recently commited reachability pattern of the local socket status
   *  instance.  This can be used by socket status instances running in a JVM
   *  separate from the BootstrapRegistry.
   */
  public void notifyStatus(Map<InetAddress,Integer> commitedLinks)
    throws RemoteException;

  /**
   *  Returns the content of the current socket status object.  This is used
   *  to obtain the state of a socket status object running in another JVM on
   *  the local machine.
   */
  public Map<InetAddress,Integer> getStatus()
    throws RemoteException;

  /**
   *  Set the probability of a datagram packet to be correctly delivered
   *  at the host identified by <code>inet</code>.  The probability is
   *  defined as <code>prob</code> / <code>BASE</code>.
   */
  public void setStatus(InetAddress inet, int prob)
    throws RemoteException;

} // END of SocketStatus
