/*
 * Copyright (c) 1998-2002 The Jgroup Team.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package jgroup.upgrade;

import java.rmi.RemoteException;

import jgroup.core.ExternalGMIListener;
import jgroup.core.arm.ExecException;
import jgroup.core.arm.UnknownGroupException;
import jgroup.relacs.config.AppConfig;


/**
 *  The <code>UpgradeManager</code> interface describes the methods that
 *  a upgrade management client can invoke on the upgrade manager to
 *  upgrade and downgrade object groups within the distributed system.
 *
 *  @author Hein Meling
 *  @since Jgroup 1.2
 */
public interface UpgradeManager
  extends ExternalGMIListener
{

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods for upgrade management
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Upgrade the group for the specified application, with the
   *  applications required redundancy.
   *
   *  @param app
   *    The application details required to upgrade the group.
   *  @exception RemoteException
   *    Raised if there was communication problems; only when invoked remotely.
   *  @exception UnknownGroupException
   *    Raised if the specified application has no group in the system.
   */
  public void upgradeGroup(AppConfig app)
    throws RemoteException, UnknownGroupException, ExecException;


  /**
   *  Downgrade the group for the given application.
   *
   *  @param app
   *    The application group to remove.
   *  @exception RemoteException
   *    Raised if there was communication problems; only when invoked remotely.
   *  @exception UnknownGroupException
   *    Raised if the specified application has no group in the system.
   */
  public void downgradeGroup(AppConfig app)
    throws RemoteException, UnknownGroupException;


} // END UpgradeManager
