/*
 * Copyright (c) 1998-2002 The Jgroup Team.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package jgroup.upgrade;

import java.rmi.RemoteException;

import jgroup.core.ExternalGMIListener;
import jgroup.core.Layer;
import jgroup.core.arm.ExecException;
import jgroup.core.arm.UnknownGroupException;
import jgroup.relacs.config.AppConfig;

/**
 *  Interface implemented by the <code>UpgradeLayer</code> to enable the
 *  UpgradeManager to initiate an upgrade.
 *
 *  Since the <code>UpgradeLayer</code> uses the EGMI approach for
 *  soliciting upgrade requests, this interface needs to extend the EGMI
 *  listener.
 *
 *  @author Hein Meling
 *  @since Jgroup 1.2
 */
public interface UpgradeService
  extends Layer, ExternalGMIListener
{

  /////////////////////////////////////////////////////////////////////////////
  // Constants
  /////////////////////////////////////////////////////////////////////////////

  /**
   *  Service name used to register the upgrade service layer in the
   *  dependable registry.
   */
  public static final String UPGRADE_SERVICE = "UpgradeService";


  /////////////////////////////////////////////////////////////////////////////
  // Upgrade related methods
  /////////////////////////////////////////////////////////////////////////////

  public void upgradeRequest(AppConfig newApp)
    throws RemoteException, UnknownGroupException, ExecException;

}
