/*
 * Copyright (c) 1998-2002 The Jgroup Team.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package jgroup.util;


/**
 *  The <code> BooleanSet </code> class implements a set through an array
 *  of bits.
 *
 *  @author  Alberto Montresor
 *  @author  Salvatore Cammarata
 *  @since Jgroup 1.2
 */
public class BooleanSet 
{
  
  /////////////////////////////////////////////////////////////////////////////
  // Fields
  /////////////////////////////////////////////////////////////////////////////

  private boolean[] set;
  private int size;
  
  
  /////////////////////////////////////////////////////////////////////////////
  // Constructors
  /////////////////////////////////////////////////////////////////////////////

  /**
   *  Builds an empty bit set 
   */
  public BooleanSet(int size)
  {
    this.size = 0;
    set = new boolean[size];
  }

  
  /////////////////////////////////////////////////////////////////////////////
  // Methods
  /////////////////////////////////////////////////////////////////////////////

  /**
   *  Set the i-th element to true
   */
  public void set(int index)
  {
    if (index >= set.length) {
      doubleCapacity();
    }
    if (!set[index]) {
      set[index] = true;
      size++;
    }
  }

  /**
   *  Set the i-th element to false
   */
  public void unset(int index)
  {
    if (index >= set.length) {
      doubleCapacity();
    }
    if (set[index]) {
      set[index] = false;
      size--;
    }
  }

  /**
   *  Get the i-th element
   */
  public boolean lookup(int index)
  { 
    return set[index];
  }

  /**
   *  Return the number of elements set to true
   */
  public int size()
  {
    return size;
  }

  /**
   *  Return the capacity of the bit set
   */
  public int lastIndex()
  {
    return set.length;
  }

  /**
   *  Clear the bit set
   */
  public void clear()
  {
    for (int i=0; i < set.length; i++)
      set[i] = false;
    size = 0;
  }

  /**
   *  Returns a string representation of this object
   */
  public String toString()
  {
    StringBuilder ret = new StringBuilder();
    for(int i=0; i <= set.length; i++)
      ret.append(set[i] ? "1" : "0");
    return ret.toString(); 
  }

  /////////////////////////////////////////////////////////////////////////////
  // Private methods
  /////////////////////////////////////////////////////////////////////////////

  private void doubleCapacity()
  {
    // FIXME Write this method
    throw new UnsupportedOperationException();
  }
  
} // END BooleanSet

