/*
 * Copyright (c) 1998-2005 The Jgroup Team.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */
package jgroup.util.log;

import jgroup.relacs.config.Domain;

/**
 * A partition or merge event to be logged.
 * 
 * @author Hein Meling
 */
public class DomainEvent
  extends Event
{

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constants
  ////////////////////////////////////////////////////////////////////////////////////////////

  private static final long serialVersionUID = 227099539664502309L;

  /**
   * Supported domain event types.
   */
  public enum Type implements EventType { Partition, Merge }


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Fields
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   * The domain name for this DomainEvent
   */
  private String domain;


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constructor
  ////////////////////////////////////////////////////////////////////////////////////////////

  public DomainEvent(EventType type, String description, Domain domain)
  {
    super(type, description);
    this.domain = domain.getName();
  }


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Access Methods
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   * @return Returns the domain.
   */
  public String getDomain()
  {
    return domain;
  }


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods from Object
  ////////////////////////////////////////////////////////////////////////////////////////////

  public String toString()
  {
    StringBuilder buf = commonToString(true);
    buf.append(" domain=");
    buf.append(domain);
    return buf.toString();
  }

} // END DomainEvent
